@@ -1,27 +1,14 @@
-/*  This file is part of the Kate project.
- *
- *  Copyright (C) 2010 Christoph Cullmann <cullmann@kde.org>
- *
- *  Based on code of the SmartCursor/Range by:
- *  Copyright (C) 2003-2005 Hamish Rodda <rodda@kde.org>
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Library General Public
- *  License as published by the Free Software Foundation; either
- *  version 2 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Library General Public License for more details.
- *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
- */
+/*
+    SPDX-FileCopyrightText: 2010 Christoph Cullmann <cullmann@kde.org>
+
+    Based on code of the SmartCursor/Range by:
+    SPDX-FileCopyrightText: 2003-2005 Hamish Rodda <rodda@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #include "katetextrange.h"
+#include "katedocument.h"
 #include "katetextbuffer.h"
 
 namespace Kate
@@ -40,52 +27,43 @@ TextRange::TextRange(TextBuffer &buffer, const KTextEditor::Range &range, Insert
     m_buffer.m_ranges.insert(this);
 
     // check if range now invalid, there can happen no feedback, as m_feedback == 0
-    checkValidity();
+    // only place where KTextEditor::LineRange::invalid() for old range makes sense, as we were yet not registered!
+    checkValidity(KTextEditor::LineRange::invalid());
 }
 
 TextRange::~TextRange()
 {
-    /**
-     * reset feedback, don't want feedback during destruction
-     */
+    // reset feedback, don't want feedback during destruction
     m_feedback = nullptr;
 
     // remove range from m_ranges
-    fixLookup(m_start.line(), m_end.line(), -1, -1);
+    fixLookup(toLineRange(), KTextEditor::LineRange::invalid());
 
     // remove this range from the buffer
     m_buffer.m_ranges.remove(this);
 
-    /**
-     * trigger update, if we have attribute
-     * notify right view
-     * here we can ignore feedback, even with feedback, we want none if the range is deleted!
-     */
+    // trigger update, if we have attribute
+    // notify right view
+    // here we can ignore feedback, even with feedback, we want none if the range is deleted!
     if (m_attribute) {
-        m_buffer.notifyAboutRangeChange(m_view, m_start.line(), m_end.line(), true /* we have a attribute */);
+        m_buffer.notifyAboutRangeChange(m_view, toLineRange(), true /* we have a attribute */);
     }
 }
 
 void TextRange::setInsertBehaviors(InsertBehaviors _insertBehaviors)
 {
-    /**
-     * nothing to do?
-     */
+    // nothing to do?
     if (_insertBehaviors == insertBehaviors()) {
         return;
     }
 
-    /**
-     * modify cursors
-     */
+    // modify cursors
     m_start.setInsertBehavior((_insertBehaviors & ExpandLeft) ? KTextEditor::MovingCursor::StayOnInsert : KTextEditor::MovingCursor::MoveOnInsert);
     m_end.setInsertBehavior((_insertBehaviors & ExpandRight) ? KTextEditor::MovingCursor::MoveOnInsert : KTextEditor::MovingCursor::StayOnInsert);
 
-    /**
-     * notify world
-     */
+    // notify world
     if (m_attribute || m_feedback) {
-        m_buffer.notifyAboutRangeChange(m_view, m_start.line(), m_end.line(), true /* we have a attribute */);
+        m_buffer.notifyAboutRangeChange(m_view, toLineRange(), true /* we have a attribute */);
     }
 }
 
@@ -106,22 +84,16 @@ KTextEditor::MovingRange::InsertBehaviors TextRange::insertBehaviors() const
 
 void TextRange::setEmptyBehavior(EmptyBehavior emptyBehavior)
 {
-    /**
-     * nothing to do?
-     */
+    // nothing to do?
     if (m_invalidateIfEmpty == (emptyBehavior == InvalidateIfEmpty)) {
         return;
     }
 
-    /**
-     * remember value
-     */
+    // remember value
     m_invalidateIfEmpty = (emptyBehavior == InvalidateIfEmpty);
 
-    /**
-     * invalidate range?
-     */
-    if (end() <= start()) {
+    // invalidate range?
+    if (endInternal() <= startInternal()) {
         setRange(KTextEditor::Range::invalid());
     }
 }
@@ -134,8 +106,7 @@ void TextRange::setRange(const KTextEditor::Range &range)
     }
 
     // remember old line range
-    int oldStartLine = m_start.line();
-    int oldEndLine = m_end.line();
+    const auto oldLineRange = toLineRange();
 
     // change start and end cursor
     m_start.setPosition(range.start());
@@ -143,7 +114,7 @@ void TextRange::setRange(const KTextEditor::Range &range)
 
     // check if range now invalid, don't emit feedback here, will be handled below
     // otherwise you can't delete ranges in feedback!
-    checkValidity(oldStartLine, oldEndLine, false);
+    checkValidity(oldLineRange, false);
 
     // no attribute or feedback set, be done
     if (!m_attribute && !m_feedback) {
@@ -151,21 +122,19 @@ void TextRange::setRange(const KTextEditor::Range &range)
     }
 
     // get full range
-    int startLineMin = oldStartLine;
-    if (oldStartLine == -1 || (m_start.line() != -1 && m_start.line() < oldStartLine)) {
+    int startLineMin = oldLineRange.start();
+    if (oldLineRange.start() == -1 || (m_start.lineInternal() != -1 && m_start.lineInternal() < oldLineRange.start())) {
         startLineMin = m_start.line();
     }
 
-    int endLineMax = oldEndLine;
-    if (oldEndLine == -1 || m_end.line() > oldEndLine) {
-        endLineMax = m_end.line();
+    int endLineMax = oldLineRange.end();
+    if (oldLineRange.end() == -1 || m_end.lineInternal() > oldLineRange.end()) {
+        endLineMax = m_end.lineInternal();
     }
 
-    /**
-     * notify buffer about attribute change, it will propagate the changes
-     * notify right view
-     */
-    m_buffer.notifyAboutRangeChange(m_view, startLineMin, endLineMax, m_attribute);
+    // notify buffer about attribute change, it will propagate the changes
+    // notify right view
+    m_buffer.notifyAboutRangeChange(m_view, {startLineMin, endLineMax}, m_attribute);
 
     // perhaps need to notify stuff!
     if (m_feedback) {
@@ -178,34 +147,28 @@ void TextRange::setRange(const KTextEditor::Range &range)
     }
 }
 
-void TextRange::checkValidity(int oldStartLine, int oldEndLine, bool notifyAboutChange)
+void TextRange::checkValidity(KTextEditor::LineRange oldLineRange, bool notifyAboutChange)
 {
-    /**
-     * in any case: reset the flag, to avoid multiple runs
-     */
+    // in any case: reset the flag, to avoid multiple runs
     m_isCheckValidityRequired = false;
 
-    /**
-     * check if any cursor is invalid or the range is zero size and it should be invalidated then
-     */
+    // check if any cursor is invalid or the range is zero size and it should be invalidated then
     if (!m_start.isValid() || !m_end.isValid() || (m_invalidateIfEmpty && m_end <= m_start)) {
         m_start.setPosition(-1, -1);
         m_end.setPosition(-1, -1);
     }
 
-    /**
-     * for ranges which are allowed to become empty, normalize them, if the end has moved to the front of the start
-     */
+    // for ranges which are allowed to become empty, normalize them, if the end has moved to the front of the start
     if (!m_invalidateIfEmpty && m_end < m_start) {
         m_end.setPosition(m_start);
     }
 
     // fix lookup
-    fixLookup(oldStartLine, oldEndLine, m_start.line(), m_end.line());
+    fixLookup(oldLineRange, toLineRange());
 
     // perhaps need to notify stuff!
     if (notifyAboutChange && m_feedback) {
-        m_buffer.notifyAboutRangeChange(m_view, m_start.line(), m_end.line(), false /* attribute not interesting here */);
+        m_buffer.notifyAboutRangeChange(m_view, toLineRange(), false /* attribute not interesting here */);
 
         // do this last: may delete this range
         if (!toRange().isValid()) {
@@ -216,39 +179,39 @@ void TextRange::checkValidity(int oldStartLine, int oldEndLine, bool notifyAbout
     }
 }
 
-void TextRange::fixLookup(int oldStartLine, int oldEndLine, int startLine, int endLine)
+void TextRange::fixLookup(KTextEditor::LineRange oldLineRange, KTextEditor::LineRange lineRange)
 {
     // nothing changed?
-    if (oldStartLine == startLine && oldEndLine == endLine) {
+    if (oldLineRange == lineRange) {
         return;
     }
 
     // now, not both can be invalid
-    Q_ASSERT(oldStartLine >= 0 || startLine >= 0);
-    Q_ASSERT(oldEndLine >= 0 || endLine >= 0);
+    Q_ASSERT(oldLineRange.start() >= 0 || lineRange.start() >= 0);
+    Q_ASSERT(oldLineRange.end() >= 0 || lineRange.end() >= 0);
 
     // get full range
-    int startLineMin = oldStartLine;
-    if (oldStartLine == -1 || (startLine != -1 && startLine < oldStartLine)) {
-        startLineMin = startLine;
+    int startLineMin = oldLineRange.start();
+    if (oldLineRange.start() == -1 || (lineRange.start() != -1 && lineRange.start() < oldLineRange.start())) {
+        startLineMin = lineRange.start();
     }
 
-    int endLineMax = oldEndLine;
-    if (oldEndLine == -1 || endLine > oldEndLine) {
-        endLineMax = endLine;
+    int endLineMax = oldLineRange.end();
+    if (oldLineRange.end() == -1 || lineRange.end() > oldLineRange.end()) {
+        endLineMax = lineRange.end();
     }
 
     // get start block
-    int blockIndex = m_buffer.blockForLine(startLineMin);
-    Q_ASSERT(blockIndex >= 0);
+    int blockIdx = m_buffer.blockForLine(startLineMin);
+    Q_ASSERT(blockIdx >= 0);
+
+    size_t blockIndex = blockIdx;
 
     // remove this range from m_ranges
     for (; blockIndex < m_buffer.m_blocks.size(); ++blockIndex) {
-        // get block
-        TextBlock *block = m_buffer.m_blocks[blockIndex];
-
         // either insert or remove range
-        if ((endLine < block->startLine()) || (startLine >= (block->startLine() + block->lines()))) {
+        TextBlock *block = m_buffer.m_blocks[blockIndex];
+        if ((lineRange.end() < block->startLine()) || (lineRange.start() >= (block->startLine() + block->lines()))) {
             block->removeRange(this);
         } else {
             block->updateRange(this);
@@ -266,89 +229,70 @@ void TextRange::fixLookup(int oldStartLine, int oldEndLine, int startLine, int e
 
 void TextRange::setView(KTextEditor::View *view)
 {
-    /**
-     * nothing changes, nop
-     */
+    // nothing changes, nop
     if (view == m_view) {
         return;
     }
 
-    /**
-     * remember the new attribute
-     */
+    // remember the new attribute
     m_view = view;
 
-    /**
-     * notify buffer about attribute change, it will propagate the changes
-     * notify all views (can be optimized later)
-     */
+    // notify buffer about attribute change, it will propagate the changes
+    // notify all views (can be optimized later)
     if (m_attribute || m_feedback) {
-        m_buffer.notifyAboutRangeChange(nullptr, m_start.line(), m_end.line(), m_attribute);
+        m_buffer.notifyAboutRangeChange(nullptr, toLineRange(), m_attribute);
     }
 }
 
 void TextRange::setAttribute(KTextEditor::Attribute::Ptr attribute)
 {
-    /**
-     * remember the new attribute
-     */
+    // nothing changes, nop, only pointer compare
+    if (attribute == m_attribute) {
+        return;
+    }
+
+    // remember the new attribute
     m_attribute = attribute;
 
-    /**
-     * notify buffer about attribute change, it will propagate the changes
-     * notify right view
-     */
-    m_buffer.notifyAboutRangeChange(m_view, m_start.line(), m_end.line(), m_attribute);
+    // notify buffer about attribute change, it will propagate the changes
+    // notify right view
+    m_buffer.notifyAboutRangeChange(m_view, toLineRange(), true /* even for nullptr attribute, we had before one => repaint */);
 }
 
 void TextRange::setFeedback(KTextEditor::MovingRangeFeedback *feedback)
 {
-    /**
-     * nothing changes, nop
-     */
+    // nothing changes, nop
     if (feedback == m_feedback) {
         return;
     }
 
-    /**
-     * remember the new feedback object
-     */
+    // remember the new feedback object
     m_feedback = feedback;
 
-    /**
-     * notify buffer about feedback change, it will propagate the changes
-     * notify right view
-     */
-    m_buffer.notifyAboutRangeChange(m_view, m_start.line(), m_end.line(), m_attribute);
+    // notify buffer about feedback change, it will propagate the changes
+    // notify right view
+    m_buffer.notifyAboutRangeChange(m_view, toLineRange(), m_attribute);
 }
 
 void TextRange::setAttributeOnlyForViews(bool onlyForViews)
 {
-    /**
-     * just set the value, no need to trigger updates, printing is not interruptable
-     */
+    // just set the value, no need to trigger updates, printing is not interruptable
     m_attributeOnlyForViews = onlyForViews;
 }
 
 void TextRange::setZDepth(qreal zDepth)
 {
-    /**
-     * nothing changes, nop
-     */
+    // nothing changes, nop
     if (zDepth == m_zDepth) {
         return;
     }
 
-    /**
-     * remember the new attribute
-     */
+    // remember the new attribute
     m_zDepth = zDepth;
 
-    /**
-     * notify buffer about attribute change, it will propagate the changes
-     */
+    // notify buffer about attribute change, it will propagate the changes
     if (m_attribute) {
-        m_buffer.notifyAboutRangeChange(m_view, m_start.line(), m_end.line(), m_attribute);
+        m_buffer.notifyAboutRangeChange(m_view, toLineRange(), m_attribute);
     }
 }
 
