@@ -99,6 +99,8 @@ public:
 
     KTextEditor::Range rangeOnLine(KTextEditor::Range range, int line) const;
 
+    void setMetaData(const KPluginMetaData &metaData);
+
 private:
     void showAndSetOpeningErrorAccess();
     /*
@@ -199,7 +201,7 @@ Q_SIGNALS:
      * \param range range that the newly inserted text occupies
      * \see insertText(), insertLine()
      */
-    void textInserted(KTextEditor::Document *document, const KTextEditor::Range &range);
+    void textInsertedRange(KTextEditor::Document *document, const KTextEditor::Range &range);
 
     /**
      * The \p document emits this signal whenever \p range was removed, i.e.
@@ -520,9 +522,6 @@ public:
      */
     void writeSessionConfig(KConfigGroup &config, const QSet<QString> &flags = QSet<QString>()) override;
 
-Q_SIGNALS:
-    void configChanged();
-
     //
     // KTextEditor::MarkInterface
     //
@@ -631,7 +630,7 @@ public:
     virtual void setVariable(const QString &name, const QString &value);
 
 private:
-    QMap<QString, QString> m_storedVariables;
+    std::map<QString, QString> m_storedVariables;
 
     //
     // MovingInterface API
@@ -643,7 +642,8 @@ public:
      * @param insertBehavior insertion behavior
      * @return new moving cursor for the document
      */
-    KTextEditor::MovingCursor *newMovingCursor(const KTextEditor::Cursor &position, KTextEditor::MovingCursor::InsertBehavior insertBehavior = KTextEditor::MovingCursor::MoveOnInsert) override;
+    KTextEditor::MovingCursor *newMovingCursor(const KTextEditor::Cursor &position,
+                                               KTextEditor::MovingCursor::InsertBehavior insertBehavior = KTextEditor::MovingCursor::MoveOnInsert) override;
 
     /**
      * Create a new moving range for this document.
@@ -688,7 +688,10 @@ public:
      * @param fromRevision from this revision we want to transform
      * @param toRevision to this revision we want to transform, default of -1 is current revision
      */
-    void transformCursor(KTextEditor::Cursor &cursor, KTextEditor::MovingCursor::InsertBehavior insertBehavior, qint64 fromRevision, qint64 toRevision = -1) override;
+    void transformCursor(KTextEditor::Cursor &cursor,
+                         KTextEditor::MovingCursor::InsertBehavior insertBehavior,
+                         qint64 fromRevision,
+                         qint64 toRevision = -1) override;
 
     /**
      * Transform a cursor from one revision to an other.
@@ -698,7 +701,8 @@ public:
      * @param fromRevision from this revision we want to transform
      * @param toRevision to this revision we want to transform, default of -1 is current revision
      */
-    void transformCursor(int &line, int &column, KTextEditor::MovingCursor::InsertBehavior insertBehavior, qint64 fromRevision, qint64 toRevision = -1) override;
+    void
+    transformCursor(int &line, int &column, KTextEditor::MovingCursor::InsertBehavior insertBehavior, qint64 fromRevision, qint64 toRevision = -1) override;
 
     /**
      * Transform a range from one revision to an other.
@@ -708,7 +712,11 @@ public:
      * @param fromRevision from this revision we want to transform
      * @param toRevision to this revision we want to transform, default of -1 is current revision
      */
-    void transformRange(KTextEditor::Range &range, KTextEditor::MovingRange::InsertBehaviors insertBehaviors, KTextEditor::MovingRange::EmptyBehavior emptyBehavior, qint64 fromRevision, qint64 toRevision = -1) override;
+    void transformRange(KTextEditor::Range &range,
+                        KTextEditor::MovingRange::InsertBehaviors insertBehaviors,
+                        KTextEditor::MovingRange::EmptyBehavior emptyBehavior,
+                        qint64 fromRevision,
+                        qint64 toRevision = -1) override;
 
     //
     // MovingInterface Signals
@@ -721,8 +729,8 @@ Q_SIGNALS:
     void aboutToDeleteMovingInterfaceContent(KTextEditor::Document *document);
 
     /**
-     * This signal is emitted before the ranges of a document are invalidated and the revisions are deleted as the document is cleared (for example on load/reload).
-     * While this signal is emitted, still the old document content is around before the clear.
+     * This signal is emitted before the ranges of a document are invalidated and the revisions are deleted as the document is cleared (for example on
+     * load/reload). While this signal is emitted, still the old document content is around before the clear.
      * @param document the document which the interface belongs too which will invalidate its data
      */
     void aboutToInvalidateMovingInterfaceContent(KTextEditor::Document *document);
@@ -808,7 +816,8 @@ public:
     KateHighlighting *highlight() const;
 
 public Q_SLOTS:
-    void tagLines(int start, int end);
+    void tagLines(KTextEditor::LineRange lineRange);
+    void tagLine(int line);
 
 private Q_SLOTS:
     void internalHlChanged();
@@ -837,6 +846,7 @@ public:
     void backspace(KTextEditor::ViewPrivate *view, const KTextEditor::Cursor &);
     void del(KTextEditor::ViewPrivate *view, const KTextEditor::Cursor &);
     void transpose(const KTextEditor::Cursor &);
+    void swapTextRanges(KTextEditor::Range firstWord, KTextEditor::Range secondWord);
     void paste(KTextEditor::ViewPrivate *view, const QString &text);
 
 public:
@@ -1150,11 +1160,11 @@ public:
 public:
     KateDocumentConfig *config()
     {
-        return m_config;
+        return m_config.get();
     }
     KateDocumentConfig *config() const
     {
-        return m_config;
+        return m_config.get();
     }
 
     void updateConfig();
@@ -1162,7 +1172,7 @@ public:
 private:
     void makeAttribs(bool needInvalidate = true);
 
-    KateDocumentConfig *const m_config;
+    std::unique_ptr<KateDocumentConfig> const m_config;
 
     /**
      * Variable Reader
@@ -1276,7 +1286,9 @@ public:
      * The first OffsetList is from decoded to encoded, and the second OffsetList from
      * encoded to decoded.
      **/
-    QString decodeCharacters(const KTextEditor::Range &range, KTextEditor::DocumentPrivate::OffsetList &decToEncOffsetList, KTextEditor::DocumentPrivate::OffsetList &encToDecOffsetList);
+    QString decodeCharacters(const KTextEditor::Range &range,
+                             KTextEditor::DocumentPrivate::OffsetList &decToEncOffsetList,
+                             KTextEditor::DocumentPrivate::OffsetList &encToDecOffsetList);
     void replaceCharactersByEncoding(const KTextEditor::Range &range);
 
 protected:
@@ -1419,7 +1431,7 @@ private:
     /**
      * current autobrace range
      */
-    QSharedPointer<KTextEditor::MovingRange> m_currentAutobraceRange;
+    std::unique_ptr<KTextEditor::MovingRange> m_currentAutobraceRange;
     /**
      * current autobrace closing character (e.g. ']')
      */
@@ -1434,12 +1446,6 @@ public:
 Q_SIGNALS:
     void loaded(KTextEditor::DocumentPrivate *document);
 
-private Q_SLOTS:
-    /**
-     * trigger a close of this document in the application
-     */
-    void closeDocumentInApplication();
-
 private:
     // To calculate a QHash.keys() is quite expensive,
     // better keep a copy of that list updated when a view is added or removed.
